@@ -1,10 +1,11 @@
 #!/usr/bin/env python3
 import json
+import sys
 import time
 import requests
 import argparse, argcomplete
 import logging
-from PM3.model.process import Process, ProcessStatus, ProcessStatusLight
+from PM3.model.process import Process, ProcessStatus, ProcessStatusLight, ProcessList
 from PM3.model.pm3_protocol import RetMsg
 from rich import print
 from rich.table import Table
@@ -18,11 +19,17 @@ from pytailer import async_fail_tail
 
 #logging.basicConfig(level=logging.DEBUG)
 
-async def tailfile(f):
-    with async_fail_tail(f, lines=10) as tail:
+async def tailfile(f, lines=10):
+    with async_fail_tail(f, lines=lines) as tail:
         async for line in tail:  # be careful: infinite loop!
             print(line, end='', flush=True)
 
+def _clean_ls_proc(p: dict) -> dict:
+    p.pop('pid')
+    p.pop('restart')
+    p.pop('pm3_home')
+    return p
+
 def _setup():
     pm3_home_dir = Path('~/.pm3').expanduser()
     config_file = f'{pm3_home_dir}/config.ini'
@@ -39,7 +46,6 @@ def _setup():
             else:
                 cmd = '#pm3_backend not found'
 
-
     if not Path(config_file).is_file():
         config = ConfigParser()
         config['main_section'] = {
@@ -121,10 +127,12 @@ def _post(path, jdata):
     else:
         return RetMsg(err=True, msg='Connection Error')
 
-def _parse_retmsg_payload(res: RetMsg):
+#def _parse_retmsg_payload(res: RetMsg):
+def _parse_retmsg(res: RetMsg):
     if res.err:
         print(f"[red]{res.msg}[/red]")
-    else:
+
+    if res.payload:
         for pi in res.payload:
             pi = RetMsg(**pi)
             if pi.err:
@@ -134,6 +142,13 @@ def _parse_retmsg_payload(res: RetMsg):
             else:
                 print(f"[green]{pi.msg}[/green]")
 
+    else:
+        if res.warn:
+            print(f"[yellow]{res.msg}[/yellow]")
+        else:
+            print(f"[green]{res.msg}[/green]")
+
+
 def _ls(id_or_name='all', format='table'):
     res = _get(f'ls/{id_or_name}')
     if res:
@@ -196,7 +211,8 @@ def _tabulate_ls(data):
     table = Table(show_header=True, header_style="bold yellow")
 
     for n, r in enumerate(data):
-        r = Process(**r).dict()  # Validate and sort
+        #r = Process(**r).dict()  # Validate and sort
+        r = ProcessList(**r).dict()  # Validate and sort
         if n == 0:
             for h in r.keys():
                 table.add_column(h)
@@ -288,10 +304,23 @@ def main():
     parser_rename.add_argument('-n', '--name', dest='pm3_name', help='name into pm3', required=True)
 
     parser_log = subparsers.add_parser('log', help='show log for a process')
-    parser_log.add_argument('id_or_name', help='Show Log for id or process name')
+    parser_log.add_argument('id_or_name', const='all', nargs='?', type=str, help='id or process name')
+    parser_log.add_argument('-f', '--follow', action='store_true', help='tail follow')
+    parser_log.add_argument('-n', '--lines', const=10, default=10, nargs='?', type=int, help='how many lines')
 
     parser_err = subparsers.add_parser('err', help='show log for a process')
-    parser_err.add_argument('id_or_name', help='Show Errors for id or process name')
+    parser_err.add_argument('id_or_name', const='all', nargs='?', type=str, help='id or process name')
+    parser_err.add_argument('-f', '--follow', action='store_true', help='tail follow')
+    parser_err.add_argument('-n', '--lines', const=10, default=10, nargs='?', type=int, help='how many lines')
+
+    parser_dump = subparsers.add_parser('dump', help='dump process in file')
+    parser_dump.add_argument('id_or_name', const='all', nargs='?', type=str, help='id or process name')
+    parser_dump.add_argument('-f', '--file', dest='dump_file', help='dump into file', required=False)
+
+    parser_load = subparsers.add_parser('load', help='load process from a file')
+    parser_load.add_argument('-f', '--file', dest='load_file', help='load from this file', required=True)
+    parser_load.add_argument('-r', '--rewrite', dest='load_rewrite', action='store_true', help='rewrite if process already exist')
+    parser_load.add_argument('-y', '--yes', dest='load_yes', action='store_true', help='response always yes')
 
     argcomplete.autocomplete(parser)
     args = parser.parse_args()
@@ -379,40 +408,89 @@ def main():
 
     elif args.subparser == 'rm':
         res = _get(f'rm/{args.id_or_name}')
-        _parse_retmsg_payload(res)
+        _parse_retmsg(res)
 
     elif args.subparser == 'start':
         res = _get(f'start/{args.id_or_name}')
-        _parse_retmsg_payload(res)
+        _parse_retmsg(res)
 
     elif args.subparser == 'stop':
         res = _get(f'stop/{args.id_or_name}')
-        _parse_retmsg_payload(res)
+        _parse_retmsg(res)
 
     elif args.subparser == 'restart':
         res = _get(f'restart/{args.id_or_name}')
-        _parse_retmsg_payload(res)
+        _parse_retmsg(res)
 
     elif args.subparser == 'reset':
         res = _get(f'reset/{args.id_or_name}')
-        _parse_retmsg_payload(res)
+        _parse_retmsg(res)
 
     elif args.subparser in ('log', 'err'):
-        res = _get(f'ls/{args.id_or_name}')
+        res = _get(f"ls/{args.id_or_name or 'all'}")
         if res and not res.err:
             for p in res.payload:
-                #print(p)
-                if args.subparser == 'log':
-                    ftt = p['stdout']
-                else:
-                    ftt = p['stderr']
+                ftt = p['stdout'] if args.subparser == 'log' else p['stderr']
+                print()
+
+                if not Path(ftt).is_file():
+                    print(f"[yellow] !!! file {ftt} don't exist !!! [/yellow]")
+                    continue
+
+                print(f"[yellow2] #### {ftt} #### [/yellow2]")
 
                 try:
-                    asyncio.run(tailfile(ftt))
+                    if args.follow:
+                        asyncio.run(tailfile(ftt, lines=args.lines))
+                    else:
+                        with open(ftt, "r") as f1:
+                            last_line = f1.readlines()[-args.lines:]
+                            for r in last_line:
+                                print(r, end='')
                 except KeyboardInterrupt:
                     #print('CTRL+C')
                     pass
 
+    elif args.subparser == 'dump':
+        res = _get(f"ls/{args.id_or_name or 'all'}")
+
+        if res and not res.err:
+            out = [_clean_ls_proc(p) for p in res.payload if p['pm3_id'] != 0]
+            if args.dump_file:
+                dump_file = Path(args.dump_file).as_posix()
+                if not dump_file.endswith('.json'):
+                    print('Dump file must be a .json file')
+                    sys.exit(1)
+                with open(dump_file, 'w') as f:
+                    json.dump(out, fp=f, indent=4)
+            else:
+                print(json.dumps(out, indent=4))
+
+    elif args.subparser == 'load':
+        load_file = Path(args.load_file).as_posix()
+        if not load_file.endswith('.json'):
+            print('File to load must be a json file')
+            sys.exit(1)
+
+        with open(load_file, 'r') as f:
+            prl = json.load(f)
+        for pr in prl:
+            #print(pr)
+            p = Process(**pr)
+            if args.load_yes:
+                r = 'y'
+            else:
+                r = input(f'do you want load {p.pm3_name} ({p.pm3_id}) ?')
+
+            if r == 'y':
+                post_dest = 'new' if args.load_rewrite is False else 'new/rewrite'
+                res = _post(post_dest, p.dict())
+                _parse_retmsg(res)
+            elif r == 'n':
+                print(f'[yellow]skip import {p.pm3_name} ({p.pm3_id})[/yellow]')
+            else:
+                print(f'[red]only y or n are accepted... skip[/red]')
+
     else:
         print(parser.format_help())
 
